import { InjectionToken, Directive, Optional, Inject, Input, EventEmitter, ElementRef, Output, NgModule, PLATFORM_ID } from '@angular/core';
import { switchMap, map, shareReplay, take, tap } from 'rxjs/operators';
import { BehaviorSubject, Subject, Subscription } from 'rxjs';
import { StripeService } from '@wizdm/stripe';
import { isPlatformBrowser } from '@angular/common';

/** StripeElementsModule configuration token */
const STRIPE_ELEMENTS_OPTIONS = new InjectionToken('wizdm.stripe.elements.options');
/** Relays the Elements funcitons enabling dynamic locale */
class StripeElementsDirective {
    constructor(stripe, options) {
        this.locale$ = new BehaviorSubject(undefined);
        this.elements$ = stripe.stripe$.pipe(switchMap(stripe => this.locale$.pipe(map(locale => stripe.elements(Object.assign(Object.assign({}, options), { locale }))))), shareReplay(1));
    }
    /** The Elements locale to use. The locale is automatically detected when undefined */
    set StripeElements(locale) {
        this.locale$.next(locale);
    }
    /** Creates a Stripe Element of the given type using the given locale and options */
    create(elementType, options) {
        return this.elements$.pipe(map(elements => elements.create(elementType, options)));
    }
}
StripeElementsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[StripeElements]',
                exportAs: 'StripeElements'
            },] }
];
/** @nocollapse */
StripeElementsDirective.ctorParameters = () => [
    { type: StripeService },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [STRIPE_ELEMENTS_OPTIONS,] }] }
];
StripeElementsDirective.propDecorators = {
    StripeElements: [{ type: Input }]
};

/** @dynamic
 * Base element class turning a StripeElement into an Angular component with common features
 * To be used as the base class for all the Stripe related specific components: StripeCard...
 */
class StripeElementDirective {
    constructor(elementType, elements, ref) {
        this.init$ = new Subject();
        this._options = {};
        /** Emits on value changes */
        this.valueChange = new EventEmitter();
        /** Emits when fully loaded */
        this.readyChange = new EventEmitter(true);
        /** Emits when focused */
        this.focusChange = new EventEmitter();
        /** Emits when blurred */
        this.blurChange = new EventEmitter();
        /** Emits when escape is pressed */
        this.escapeChange = new EventEmitter();
        // Ensures the element is used within the StripeElement container
        if (!elements) {
            throw new Error(`
      You're attempting to use a Stripe Element out of a proper StripeElements container.
      Make sure to wrap all the controls within a wm-stripe-elements directive.
    `);
        }
        // Creates the StripeElement observable
        this.element$ = this.init$.pipe(take(1), switchMap(options => elements.create(elementType, options).pipe(tap(elm => {
            // Disposes of the previous element instance, if any
            this._instance && this._instance.destroy();
            // Hooks on the common element's events
            elm.on('ready', () => { this.readyChange.emit(this._ready = true); });
            elm.on('focus', () => { this._focused = true; this.focusChange.emit(); });
            elm.on('blur', () => { this._focused = false; this.blurChange.emit(); });
            // Mounts the element on the DOM
            elm.mount(ref.nativeElement);
            // Keeps track of the current StripeElement instance
            return this._instance = elm;
        }))), shareReplay(1));
        // Subscribes to the element's observable waiting for the init$ to emit
        this.sub = this.element$.subscribe();
    }
    /** The StripeElement instance */
    get instance() { return this._instance; }
    /** The latest change value */
    get value() { return this._value; }
    /** True whenever the element is empty */
    get empty() { return !this._value || this._value.empty; }
    /** True whenever the element is complete and valid */
    get complete() { return !!this._value && this._value.complete; }
    /** The StripeError or null */
    get error() { return !!this._value && this._value.error || null; }
    /** True whenever the element is fully loaded */
    get ready() { return !!this._ready; }
    /** True whenever the element is focused */
    get focused() { return !!this._focused; }
    /** True whenever the element is disabled */
    get disabled() { return false; }
    /** Returns the current options object */
    get options() { return this._options || (this._options = {}); }
    /** Returns the current option classes object, if any */
    get classes() { return this._options.classes || (this._options.classes = {}); }
    /** Returns the current option style object, if any */
    get style() { return this._options.style || (this._options.style = {}); }
    /** Initialize the StripeElement with the given options */
    init(options) {
        // Pushes the initial element options object triggering the element$ Observable to emit
        this.init$.next(options);
    }
    /** Forwards the given call to the StripeElement */
    forward(fn) {
        // Short circuits with the current instance, if any
        if (this.instance) {
            return fn(this.instance);
        }
        // Resolves the observable to get the latest instance otherwise
        this.element$.pipe(take(1)).subscribe(instance => fn(instance));
    }
    /** Disables the element whenever possible. Ovverrides this default implementation to forward the status down the Element */
    disable(_) { }
    /** Focuses the element */
    focus() { this.forward(instance => instance.focus()); }
    /** Blurs the element */
    blur() { this.forward(instance => instance.blur()); }
    /** Clears the element */
    clear() { this.forward(instance => instance.clear()); }
    /** Class applied to the StripeElement's container. Defaults to StripeElement */
    set classBase(value) { this.classes.base = value; }
    /** The class name to apply when the Element is complete. Defaults to StripeElement--complete */
    set classComplete(value) { this.classes.complete = value; }
    /** The class name to apply when the Element is empty. Defaults to StripeElement--empty */
    set classEmpty(value) { this.classes.empty = value; }
    /** The class name to apply when the Element has focus. Defaults to StripeElement--focus */
    set classFocus(value) { this.classes.focus = value; }
    /** The class name to apply when the Element is invalid. Defaults to StripeElement--invalid */
    set classInvalid(value) { this.classes.invalid = value; }
    // Disposes of the element
    ngOnDestroy() {
        this.forward(instance => instance.destroy());
        this.sub.unsubscribe();
    }
}
StripeElementDirective.decorators = [
    { type: Directive }
];
/** @nocollapse */
StripeElementDirective.ctorParameters = () => [
    { type: undefined },
    { type: StripeElementsDirective },
    { type: ElementRef }
];
StripeElementDirective.propDecorators = {
    classBase: [{ type: Input }],
    classComplete: [{ type: Input }],
    classEmpty: [{ type: Input }],
    classFocus: [{ type: Input }],
    classInvalid: [{ type: Input }],
    valueChange: [{ type: Output, args: ['change',] }],
    readyChange: [{ type: Output, args: ['ready',] }],
    focusChange: [{ type: Output, args: ['focus',] }],
    blurChange: [{ type: Output, args: ['blur',] }],
    escapeChange: [{ type: Output, args: ['escape',] }]
};
function computeBaseStyle(el) {
    // Computes the element's base style to match the given HTML element
    const computed = window === null || window === void 0 ? void 0 : window.getComputedStyle(el);
    return {
        color: computed.color,
        fontFamily: computed.fontFamily,
        fontSize: computed.fontSize,
        fontStyle: computed.fontStyle,
        fontVariant: computed.fontVariant,
        fontWeight: computed.fontWeight,
        letterSpacing: computed.letterSpacing,
        textDecoration: computed.textDecoration,
        textShadow: computed.textShadow,
        textTransform: computed.textTransform
    };
}

/** @dynamic
 * Base class bridging the StripeElement with the Angular's form API implementing both a ControlValueAccessor
 * and a sync Validator enabling the use with FormControl
 */
class StripeControlDirective {
    constructor(element) {
        this.element = element;
        this.sub = new Subscription();
    }
    ngOnDestroy() { this.sub.unsubscribe(); }
    /**
     * Called by the forms API to write to the view when programmatic changes from model to view are requested.
     * NOTE: Only clearing the control is allowed
     */
    writeValue(value) {
        if (!value) {
            this.element.clear();
        }
    }
    /**
     * Function that is called by the forms API when the control status changes to or from 'DISABLED'.
     * Depending on the status, it enables or disables the appropriate DOM element.
     */
    setDisabledState(disabled) {
        this.element.disable(disabled);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * The value passed along the FormControl is the stripe Element instance to be used in the
     * payment_method object to setup or confirm the payment.
     */
    registerOnChange(fn) {
        this.sub.add(this.element.valueChange.subscribe(value => fn(value.complete ? this.element.instance : null)));
    }
    /** Registers a callback function is called by the forms API on initialization to update the form model on blur. */
    registerOnTouched(fn) {
        this.sub.add(this.element.blurChange.subscribe(() => fn()));
    }
    /** Performs synchronous validation against the provided control. */
    validate(control) {
        // Whenever the element is complete...
        if (this.element.complete) {
            // Checks for errors
            const errorType = this.element.error && this.element.error.type;
            // Translates the StripeError into a ValidationError passing along the message
            return errorType ? { [errorType]: this.element.error.message } : null;
        }
        return { required: true };
    }
}
StripeControlDirective.decorators = [
    { type: Directive }
];
/** @nocollapse */
StripeControlDirective.ctorParameters = () => [
    { type: StripeElementDirective }
];

class StripeElementsModule {
    constructor(platformId) {
        if (!isPlatformBrowser(platformId)) {
            throw new Error('StripeModule package supports Browsers only');
        }
    }
    static init(options) {
        return {
            ngModule: StripeElementsModule,
            /** Provides the global StripeElementsOptions object */
            providers: [{ provide: STRIPE_ELEMENTS_OPTIONS, useValue: options }]
        };
    }
}
StripeElementsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [StripeElementsDirective],
                exports: [StripeElementsDirective]
            },] }
];
/** @nocollapse */
StripeElementsModule.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { STRIPE_ELEMENTS_OPTIONS, StripeControlDirective, StripeElementDirective, StripeElementsDirective, StripeElementsModule, computeBaseStyle };
//# sourceMappingURL=wizdm-stripe-elements.js.map
