import { InjectionToken, Injectable, Inject, Optional, NgModule, NgZone } from '@angular/core';
import { BehaviorSubject, from, throwError } from 'rxjs';
import { switchMap, map, shareReplay, take } from 'rxjs/operators';

/** StripeCaller class implements an observable-based proxy to call the Stripe.js SDK */
class StripeCaller {
    constructor(stripejs, publicKey, options) {
        /** The optional stripe account to connect to @see [Connect](https://stripe.com/docs/connect) */
        this.account$ = new BehaviorSubject(undefined);
        // Resolves the stripe instance from the StripeContructor promise
        this.stripe$ = from(stripejs).pipe(switchMap(stripeContructor => {
            // Checks upon the stripe account to optionally connect to
            return this.account$.pipe(map(stripeAccount => stripeContructor(publicKey, Object.assign(Object.assign({}, options), { 
                // Connects to the given account, if any
                stripeAccount }))));
        }), shareReplay(1));
    }
    /**
     * For usage with [Connect](https://stripe.com/docs/connect) only.
     * Specifying a connected account ID (e.g., `acct_24BFMpJ1svR5A89k`) allows you to perform actions on behalf of that account.
     */
    connect(stripeAccount) {
        this.account$.next(stripeAccount);
    }
    /** Invokes the requested Stripe funciton */
    callStripe(signature, args) {
        // Resolves the active Stripe instance
        return this.stripe$.pipe(switchMap(stripe => {
            // Gets the requested funciton to call upon
            const fn = stripe[signature];
            // Verifies the request really belongs to a function 
            if (typeof fn !== 'function') {
                return throwError(new Error(`
        The requested signature '${signature}' is not a Stripe.js function. 
      `));
            }
            // Calls the Stripe function wrapping the result in a promise
            return Promise.resolve(fn(...args));
            // Always convert the returning value into a promise
        }), take(1)).toPromise();
    }
}

/** Stripe Public Key token */
const STRIPE_PUBLIC_KEY = new InjectionToken('wizdm.stripe.public-key');
/** Stripe Options token */
const STRIPE_OPTIONS = new InjectionToken('wizdm.stripe.options');
/** Stripe constructor token */
const STRIPEJS = new InjectionToken('wizdm.stripe.constructor');
/** Stripe.js Injectale proxy service for Angular */
/** @dynamic - tells ngc to ignore the error on potentially unknown types generated by strictEmitMetadata: true */
class StripeService extends StripeCaller {
    // Extends the StripeCaller base class enabling Stripe sdk proxy capabilities
    constructor(stripejs, publicKey, options) {
        super(stripejs, publicKey, options);
    }
    /////////////////////////////
    /// Checkout
    ///
    /// https://stripe.com/docs/js/checkout
    /////////////////////////////
    /**
     * Use `stripe.redirectToCheckout` to redirect your customers to [Checkout](https://stripe.com/docs/payments/checkout), a Stripe-hosted page to securely collect payment information.
     * When the customer completes their purchase, they are redirected back to your website.
     */
    redirectToCheckout(options) {
        return this.callStripe('redirectToCheckout', arguments);
    }
    /////////////////////////////
    /// Payment Intents
    ///
    /// https://stripe.com/docs/js/payment_intents
    /////////////////////////////
    /**
     * Use `stripe.confirmAlipayPayment` in the [Alipay Payments](https://stripe.com/docs/payments/alipay) with Payment Methods flow when the customer submits your payment form.
     * When called, it will confirm the [PaymentIntent](https://stripe.com/docs/api/payment_intents) with `data` you provide, and it will automatically redirect the customer to authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new PaymentMethod for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_alipay_payment
     */
    confirmAlipayPayment(clientSecret, data, options) {
        return this.callStripe('confirmAlipayPayment', arguments);
    }
    /**
     * Requires beta access:
     * Contact [Stripe support](https://support.stripe.com/) for more information.
     *
     * Use `stripe.confirmAuBecsDebitPayment` in the [BECS Direct Debit Payments](https://stripe.com/docs/payments/payment-methods/au-becs-debit) with Payment Methods flow when the customer submits your payment form.
     * When called, it will confirm the [PaymentIntent](https://stripe.com/docs/api/payment_intents) with `data` you provide.
     * Note that there are some additional requirements to this flow that are not covered in this reference.
     * Refer to our [integration guide](https://stripe.com/docs/payments/payment-methods/au-becs-debit-quickstart-payment-intents) for more details.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new PaymentMethod for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_au_becs_debit_payment
     */
    confirmAuBecsDebitPayment(clientSecret, data) {
        return this.callStripe('confirmAuBecsDebitPayment', arguments);
    }
    /**
     * Use `stripe.confirmBancontactPayment` in the [Bancontact Payments with Payment Methods](https://stripe.com/docs/payments/bancontact#web) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_bancontact_payment
     */
    confirmBancontactPayment(clientSecret, data, options) {
        return this.callStripe('confirmBancontactPayment', arguments);
    }
    /**
     * Use `stripe.confirmCardPayment` when the customer submits your payment form.
     * When called, it will confirm the [PaymentIntent](https://stripe.com/docs/api/payment_intents) with `data` you provide and carry out 3DS or other next actions if they are required.
     *
     * If you are using [Dynamic 3D Secure](https://stripe.com/docs/payments/3d-secure#three-ds-radar), `stripe.confirmCardPayment` will trigger your Radar rules to execute and may open a dialog for your customer to authenticate their payment.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * It can also be called with an existing `PaymentMethod`, or if you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_card_payment
     */
    confirmCardPayment(clientSecret, data, options) {
        return this.callStripe('confirmCardPayment', arguments);
    }
    ;
    /**
     * Use `stripe.confirmEpsPayment` in the [EPS Payments with Payment Methods](https://stripe.com/docs/payments/eps#web) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_eps_payment
     */
    confirmEpsPayment(clientSecret, data, options) {
        return this.callStripe('confirmEpsPayment', arguments);
    }
    /**
     * Use `stripe.confirmFpxPayment` in the [FPX Payments with Payment Methods](https://stripe.com/docs/payments/fpx#web) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_fpx_payment
     */
    confirmFpxPayment(clientSecret, data, options) {
        return this.callStripe('confirmFpxPayment', arguments);
    }
    /**
     * Use `stripe.confirmGiropayPayment` in the [giropay Payments with Payment Methods](https://stripe.com/docs/payments/giropay#web) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_giropay_payment
     */
    confirmGiropayPayment(clientSecret, data, options) {
        return this.callStripe('confirmGiropayPayment', arguments);
    }
    /**
     * Use `stripe.confirmIdealPayment` in the [iDEAL Payments with Payment Methods](https://stripe.com/docs/payments/ideal) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_ideal_payment
     */
    confirmIdealPayment(clientSecret, data, options) {
        return this.callStripe('confirmGiropayPayment', arguments);
    }
    /**
     * Use `stripe.confirmP24Payment` in the [Przelewy24 Payments with Payment Methods](https://stripe.com/docs/payments/p24#web) flow when the customer submits your payment form.
     * When called, it will confirm the `PaymentIntent` with `data` you provide, and it will automatically redirect the customer to the authorize the transaction.
     * Once authorization is complete, the customer will be redirected back to your specified `return_url`.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_p24_payment
     */
    confirmP24Payment(clientSecret, data, options) {
        return this.callStripe('confirmP24Payment', arguments);
    }
    /**
     * Use `stripe.confirmSepaDebitPayment` in the [SEPA Direct Debit Payments](https://stripe.com/docs/payments/sepa-debit) with Payment Methods flow when the customer submits your payment form.
     * When called, it will confirm the [PaymentIntent](https://stripe.com/docs/api/payment_intents) with `data` you provide.
     * Note that there are some additional requirements to this flow that are not covered in this reference.
     * Refer to our [integration guide](https://stripe.com/docs/payments/sepa-debit) for more details.
     *
     * When you confirm a `PaymentIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `PaymentIntent`, this method can automatically create and attach a new PaymentMethod for you.
     * If you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/payment_intents/confirm_sepa_debit_payment
     */
    confirmSepaDebitPayment(clientSecret, data) {
        return this.callStripe('confirmSepaDebitPayment', arguments);
    }
    /**
     * Use `stripe.handleCardAction` in the Payment Intents API [manual confirmation](https://stripe.com/docs/payments/payment-intents/web-manual) flow to handle a [PaymentIntent](https://stripe.com/docs/api/payment_intents) with the `requires_action` status.
     * It will throw an error if the `PaymentIntent` has a different status.
     *
     * Note that `stripe.handleCardAction` may take several seconds to complete.
     * During that time, you should disable your form from being resubmitted and show a waiting indicator like a spinner.
     * If you receive an error result, you should be sure to show that error to the customer, re-enable the form, and hide the waiting indicator.
     *
     * Additionally, `stripe.handleCardAction` may trigger a [3D Secure](https://stripe.com/docs/payments/3d-secure) authentication challenge.
     * The authentication challenge requires a context switch that can be hard to follow on a screen-reader.
     * Ensure that your form is accessible by ensuring that success or error messages are clearly read out.
     *
     * @docs https://stripe.com/docs/js/payment_intents/handle_card_action
     */
    handleCardAction(clientSecret) {
        return this.callStripe('handleCardAction', arguments);
    }
    /**
     * Use stripe.createPaymentMethod to convert payment information collected by elements into a [PaymentMethod](https://stripe.com/docs/api/payment_methods) object that you safely pass to your server to use in an API call.
     *
     * @docs https://stripe.com/docs/js/payment_intents/create_payment_method
     */
    createPaymentMethod(paymentMethodData) {
        return this.callStripe('createPaymentMethod', arguments);
    }
    /**
     * Retrieve a [PaymentIntent](https://stripe.com/docs/api/payment_intents) using its [client secret](https://stripe.com/docs/api/payment_intents/object#payment_intent_object-client_secret).
     *
     * @docs https://stripe.com/docs/js/payment_intents/retrieve_payment_intent
     */
    retrievePaymentIntent(clientSecret) {
        return this.callStripe('retrievePaymentIntent', arguments);
    }
    /////////////////////////////
    /// Setup Intents
    ///
    /// https://stripe.com/docs/js/setup_intents
    /////////////////////////////
    /**
     * Requires beta access:
     * Contact [Stripe support](https://support.stripe.com/) for more information.
     *
     * Use `stripe.confirmAuBecsDebitSetup` in the [BECS Direct Debit with Setup Intents](https://stripe.com/docs/payments/payment-methods/au-becs-debit-quickstart-setup-intents) flow when the customer submits your payment form.
     * When called, it will confirm the `SetupIntent` with `data` you provide.
     * Note that there are some additional requirements to this flow that are not covered in this reference.
     * Refer to our [integration guide](https://stripe.com/docs/payments/payment-methods/au-becs-debit-quickstart-setup-intents) for more details.
     *
     * When you confirm a `SetupIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `SetupIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * It can also be called with an existing `PaymentMethod`, or if you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/setup_intents/confirm_au_becs_debit_setup
     */
    confirmAuBecsDebitSetup(clientSecret, data) {
        return this.callStripe('confirmAuBecsDebitSetup', arguments);
    }
    /**
     * Use `stripe.confirmBacsDebitSetup` in the [Bacs Direct Debit Payments](https://stripe.com/docs/payments/payment-methods/bacs-debit) flow when the customer submits your payment form.
     * When called, it will confirm the [SetupIntent](https://stripe.com/docs/api/setup_intents) with `data` you provide.
     * Note that there are some additional requirements to this flow that are not covered in this reference.
     * Refer to our [integration guide](https://stripe.com/docs/payments/payment-methods/bacs-debit) for more details.
     *
     * When you confirm a `SetupIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `SetupIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * It can also be called with an existing `PaymentMethod`, or if you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     * These use cases are detailed in the sections that follow.
     *
     * @docs https://stripe.com/docs/js/setup_intents/confirm_bacs_debit_setup
     */
    confirmBacsDebitSetup(clientSecret, data) {
        return this.callStripe('confirmBacsDebitSetup', arguments);
    }
    /**
     * Use `stripe.confirmCardSetup` in the [Setup Intents API flow](https://stripe.com/docs/payments/save-and-reuse) when the customer submits your payment form.
     * When called, it will confirm the [SetupIntent](https://stripe.com/docs/api/setup_intents) with `data` you provide and carry out 3DS or other next actions if they are required.
     *
     * When you confirm a `SetupIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `SetupIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * It can also be called with an existing `PaymentMethod`, or if you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/setup_intents/confirm_card_setup
     */
    confirmCardSetup(clientSecret, data, options) {
        return this.callStripe('confirmCardSetup', arguments);
    }
    /**
     * Use `stripe.confirmSepaDebitSetup` in the [SEPA Direct Debit with Setup Intents](https://stripe.com/docs/payments/sepa-debit-setup-intents) flow when the customer submits your payment form.
     * When called, it will confirm the `SetupIntent` with `data` you provide.
     * Note that there are some additional requirements to this flow that are not covered in this reference.
     * Refer to our [integration guide](https://stripe.com/docs/payments/sepa-debit-setup-intents) for more details.
     *
     * When you confirm a `SetupIntent`, it needs to have an attached [PaymentMethod](https://stripe.com/docs/api/payment_methods).
     * In addition to confirming the `SetupIntent`, this method can automatically create and attach a new `PaymentMethod` for you.
     * It can also be called with an existing `PaymentMethod`, or if you have already attached a `PaymentMethod` you can call this method without needing to provide any additional data.
     *
     * @docs https://stripe.com/docs/js/setup_intents/confirm_sepa_debit_setup
     */
    confirmSepaDebitSetup(clientSecret, data) {
        return this.callStripe('confirmSepaDebitSetup', arguments);
    }
    /**
     * Retrieve a [SetupIntent](https://stripe.com/docs/api/setup_intents) using its client secret.
     *
     * @docs https://stripe.com/docs/js/setup_intents/retrieve_setup_intent
     */
    retrieveSetupIntent(clientSecret) {
        return this.callStripe('retrieveSetupIntent', arguments);
    }
    /////////////////////////////
    /// Payment Request
    ///
    /// https://stripe.com/docs/js/payment_request
    /////////////////////////////
    /**
     * Use `stripe.paymentRequest` to create a `PaymentRequest` object.
     * Creating a `PaymentRequest` requires that you configure it with an `options` object.
     *
     * In Safari, `stripe.paymentRequest` uses Apple Pay, and in other browsers it uses the [Payment Request API standard](https://www.w3.org/TR/payment-request/).
     */
    paymentRequest(options) {
        return this.callStripe('paymentRequest', arguments);
    }
    // createToken function implementation
    createToken() { return this.callStripe('createToken', arguments); }
    // createSource function implementation
    createSource() { return this.callStripe('createSource', arguments); }
    /**
     * Retrieve a [Source](https://stripe.com/docs/api#sources) using its unique ID and client secret.
     *
     * @docs https://stripe.com/docs/js/tokens_sources/retrieve_source
     */
    retrieveSource(source) {
        return this.callStripe('retrieveSource', arguments);
    }
}
StripeService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
StripeService.ctorParameters = () => [
    { type: Promise, decorators: [{ type: Inject, args: [STRIPEJS,] }] },
    { type: String, decorators: [{ type: Inject, args: [STRIPE_PUBLIC_KEY,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [STRIPE_OPTIONS,] }] }
];

/** Retrives the global StripeJS object  */
function getStripeJS() {
    return (!!window ? window.Stripe : undefined);
}
/** Stripe.js v3 script loader. We do not use the official loader provided
 * by @stripe/stripe-js since we may need creating multiple Stripe instances
 * while connecting to multiple accounts */
function loadStripeJS() {
    /** The official stripe-js link. The script must be loaded from the stripe server for PCI compliance */
    const STRIPE_V3_URL = 'https://js.stripe.com/v3';
    // Try to get the StripeJS instance first
    const StripeJS = getStripeJS();
    // Proceed in loading the script asyncronously
    return StripeJS ? Promise.resolve(StripeJS) : new Promise((resolve, reject) => {
        try {
            // We'll be trying to get the existing script or injecting a new one
            const findScript = () => document.querySelector(`script[src^="${STRIPE_V3_URL}"]`);
            const injectScript = () => {
                const script = document.createElement('script');
                script.src = STRIPE_V3_URL;
                script.type = 'text/javascript';
                script.async = true;
                (document.head || document.body).appendChild(script);
                return script;
            };
            // Seeks for the script elementn or inject a new one
            const script = findScript() || injectScript();
            // Listens for error or completion
            script.addEventListener('error', () => reject(new Error("Unable to load StripeJS")));
            script.addEventListener('load', () => resolve(getStripeJS()));
        }
        catch (e) {
            reject(e);
        }
    });
}

/** @dynamic - tells ngc to ignore the error on potentially unknown types generated by strictEmitMetadata: true */
class StripeModule {
    constructor(publicKey, zone) {
        if (!publicKey) {
            throw new Error(`
      Stripe module has not been initialized.
      Make sure to call StripeModule.init('pk_xxxxxxxxx') in your root or feature module.
    `);
        }
        // Triggers the stripe.js API loading asyncronously.
        if (!StripeModule._stripejs) {
            // Loads the Stripe.js avoiding to trigger change detection     
            StripeModule._stripejs = zone.runOutsideAngular(() => loadStripeJS());
        }
    }
    static init(publicKey, options) {
        return {
            ngModule: StripeModule,
            providers: [
                /** Provides the global Stripe public key */
                { provide: STRIPE_PUBLIC_KEY, useValue: publicKey },
                /** Provides the global stripe options */
                { provide: STRIPE_OPTIONS, useValue: options },
                /** Provides StripeJS as an injectable promise */
                { provide: STRIPEJS, useFactory: () => StripeModule._stripejs }
            ]
        };
    }
}
StripeModule.decorators = [
    { type: NgModule, args: [{
                providers: [StripeService]
            },] }
];
/** @nocollapse */
StripeModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [STRIPE_PUBLIC_KEY,] }] },
    { type: NgZone }
];

/**
 * Generated bundle index. Do not edit.
 */

export { STRIPEJS, STRIPE_OPTIONS, STRIPE_PUBLIC_KEY, StripeModule, StripeService, StripeCaller as ɵa };
//# sourceMappingURL=wizdm-stripe.js.map
